<!DOCTYPE html>
<!--[if lt IE 7]>      <html class="no-js lt-ie9 lt-ie8 lt-ie7"> <![endif]-->
<!--[if IE 7]>         <html class="no-js lt-ie9 lt-ie8"> <![endif]-->
<!--[if IE 8]>         <html class="no-js lt-ie9"> <![endif]-->
<!--[if gt IE 8]><!--> <html class="no-js"> <!--<![endif]-->
    <head>
	<base href="{{ base_url }}" />
        <meta charset="utf-8">
        <meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">
	<title>{% block title %}YubiAuth{% endblock %}</title>
        <meta name="description" content="">
        <meta name="viewport" content="width=device-width">
	<link rel="icon" type="image/png" href="../static/favicon.ico" />

	{% block styles %}
	<link rel="stylesheet" href="../static/css/bootstrap.min.css">
        <link rel="stylesheet" href="../static/css/bootstrap-responsive.min.css">
        <link rel="stylesheet" href="../static/css/main.css">
	{% endblock %}

        <script src="../static/js/vendor/modernizr-2.6.2-respond-1.1.0.min.js"></script>
    </head>
    <body>
        <!--[if lt IE 7]>
            <p class="chromeframe">You are using an <strong>outdated</strong> browser. Please <a href="http://browsehappy.com/">upgrade your browser</a> or <a href="http://www.google.com/chromeframe/?redirect=true">activate Google Chrome Frame</a> to improve your experience.</p>
        <![endif]-->

	{% block body %} 
	<h1>YubiAuth</h1>
	{% endblock %}

	{% block scripts %}
        <script src="//ajax.googleapis.com/ajax/libs/jquery/1.9.1/jquery.min.js"></script>
        <script>window.jQuery || document.write('<script src="../static/js/vendor/jquery-1.9.1.min.js"><\/script>')</script>

        <script src="../static/js/vendor/bootstrap.min.js"></script>

	<script src="../static/js/main.js"></script>
	{% endblock %}
    </body>
</html>
