{% extends "logged_in.html" %}

{% block title %}YubiKey: {{ yubikey.prefix }}{% endblock %}

{% block content %}
{{ super() }}

<h2>YubiKey: {{ yubikey.prefix }}</h2>

<dl class="dl-horizontal">
	<dt>Prefix</dt>
	<dd>{{ yubikey.prefix }}</dd>
	<dt>Enabled</dt>
	<dd>{{ yubikey.enabled }}</dd>
</dl>

<h3>Actions</h3>
{% if yubikey.enabled %}
<a href="yubikey/{{ yubikey.prefix }}/disable" class="btn">Disable YubiKey</a>
{% else %}
<a href="yubikey/{{ yubikey.prefix }}/enable" class="btn">Enable YubiKey</a>
{% endif %}
<br />
<a href="yubikey/{{ yubikey.prefix }}/unassign" class="btn">Unassign YubiKey</a>
<br />
<a href="yubikey/{{ yubikey.prefix }}/generate" class="btn">Generate revocation code</a>
<p>A revocation code can be used to disable a YubiKey if you are unable to access your account. Once generated, be sure to save the revocation code in a safe place. Generating a new code will invalidate any previously generated code.</p>

{% endblock %}
