{% extends "layout.html" %}

{% block title %}Session required!{% endblock %}

{% block content %}
<h1>Session required!</h1>

<p>The action you have tried to perform requires that you be logged in to the system.</p>

<a href="">Log in</a>
{% endblock %}
