{% extends "logged_in.html" %}

{% from "form.html" import render_form %}

{% block title %}Assign YubiKey{% endblock %}

{% block content %}
{{ super() }}

{{ render_form(fieldsets, 'assign_yubikey') }}

{% endblock %}
